#ifndef NEMO_CALTIME_H
#define NEMO_CALTIME_H

#include <nemo/stdint.h>
#include <nemo/unixtypes.h>
#include <nemo/tai.h>

#ifdef __cplusplus
extern "C" {
#endif

struct caltime {
  struct caldate date;
  int hour;
  int minute;
  int second;
  long offset;
};

#define CALTIME {CALDATE,0,0,0,0L}

/*
  enough space to hold:
    formatted caldate,
    ' ',
    2 digit hour, ':', 2 digit minute, ":", 2 digit seconds,
    ' ',
     '+'/'-' sign and 4 digit offset
    plus \0
*/
#define FMT_CALTIME FMT_CALDATE + 15

void	caltime_tai(const struct caltime *ct, struct tai *t);
void	caltime_utc(struct caltime *ct, const struct tai *t, int *weekday, int *yearday);  /* timezone = UTC */
void	caltime_local(struct caltime *ct, const struct tai *t, int *weekday, int *yearday);  /* timezone = local */

unsigned int	caltime_scan(struct caltime *ct, const void *in);

unsigned int	caltime_fmt(const struct caltime *ct, void *out);
unsigned int	caltime_fmt_ctime(const struct caltime *ct, void *out);
unsigned int	caltime_fmt_short(const struct caltime *ct, void *out);

unsigned int	caltime_fmt_time(const struct caltime *ct, void *out);
unsigned int	caltime_fmt_tzoffset(const struct caltime *ct, void *out);

#ifdef __cplusplus
}
#endif

#endif
