#include <nemo/stdint.h>
#include <nemo/stralloc.h>
#include <nemo/djbio.h>
#include <nemo/djbiofd.h>
#include <nemo/getln.h>
#include <nemo/fmt.h>
#include <nemo/macro_unused.h>

#include "leapsecs.h"
#include "caldate.h"
#include "caltime.h"

static const char *dayname[7] = {
  "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
};

static char out[FMT_CALTIME + FMT_ULONG];
static char x[TAI_PACK];

static stralloc line = STRALLOC;

int main(int argc __UNUSED__, char **argv __UNUSED__)
{
  struct tai t;
  struct tai t2;
  struct caltime ct;
  struct caltime ct2;
  int weekday;
  int yearday;
  double packerr;
  unsigned int match;

  if (leapsecs_init() < 0) {
    djbio_puts(djbiofd_out, "unable to init leapsecs\n");
  }

  while (!(getln(djbiofd_in, &line, &match, '\n') < 0)) {
    if (!match) break;
    if (!caltime_scan(&ct, line.s)) {
      djbio_puts(djbiofd_out, "unable to parse\n");
    }
    else {
      caltime_tai(&ct, &t);
      caltime_utc(&ct2, &t, &weekday, &yearday);
      tai_pack(&t, x);
      tai_unpack(&t2, x);
      tai_sub(&t2, &t2, &t);
      packerr = tai_approx(&t2);
      djbio_put(djbiofd_out, out, fmt_hexb(out, x, TAI_PACK));
      if (packerr) {
        djbio_put(djbiofd_out, " packerr=", 9);
        djbio_put(djbiofd_out, out, fmt_ulong(out, (unsigned long)packerr));
      }
      djbio_put(djbiofd_out, " ", 1);
      djbio_put(djbiofd_out, out, fmt_ulong0(out, (unsigned long)yearday, 3));
      djbio_put(djbiofd_out, "  ", 2);
      djbio_put(djbiofd_out, dayname[weekday], 3);
/*    printf(" %03d  %s", yearday, dayname[weekday]); */
      djbio_put(djbiofd_out, " ", 1);
      djbio_put(djbiofd_out, out, caltime_fmt(&ct2, out));
      djbio_put(djbiofd_out, "\n", 1);
    }
  }
  djbio_flush(djbiofd_out);
  return 0;
}
