#include <nemo/stdint.h>
#include <nemo/unixtypes.h>

#include <nemo/alloc.h>
#include <nemo/error.h>
#include <nemo/open.h>
#include <nemo/unix.h>

#include "leapsecs.h"

#include <sys/stat.h>
#include <fcntl.h>

struct tai *leapsecs = 0;
unsigned int leapsecs_num = 0;

int leapsecs_read(void)
{
  int fd;
  struct stat st;
  struct tai u;
  struct tai *t;
  ssize_t n;
  unsigned int i;

  fd = open_read("/etc/leapsecs.dat");
  if (fd < 0) {
    if (errno != error_noent) return -1;
    if (leapsecs) alloc_free(/*(char *)*/leapsecs);
    leapsecs = 0;
    leapsecs_num = 0;
    return 0;
  }

  if (fstat(fd, &st) < 0) {
    close(fd);
    return -1;
  }

  t = (struct tai *)alloc((unsigned long)st.st_size);
  if (!t) {
    close(fd);
    return -1;
  }

  n = read(fd, (char *)t, (unsigned long)st.st_size);
  close(fd);
  if (n != st.st_size) {
    alloc_free((char *)t);
    return -1;
  }

  n /= (ssize_t)sizeof(struct tai);

  for (i = 0; i < n; ++i) {
    tai_unpack(&u, (char *)&t[i]);
    t[i] = u;
  }

  if (leapsecs) {
    alloc_free((char *)leapsecs);
  }

  leapsecs = t;
  leapsecs_num = (unsigned int)n;

  return 0;
}
