#include <nemo/stdint.h>
#include <nemo/stralloc.h>
#include <nemo/djbio.h>
#include <nemo/djbiofd.h>
#include <nemo/getln.h>
#include <nemo/fmt.h>
#include <nemo/scan.h>
#include <nemo/str.h>

#include "caldate.h"

static const char *montab[] = {
  "January",
  "February",
  "March",
  "April",
  "May",
  "June",
  "July",
  "August",
  "September",
  "October",
  "November",
  "December"
};

static char strnum[FMT_ULONG];

int main(int argc,  char **argv)
{
  long year;
  long daystart;
  long dayend;
  long day;
  int weekday;
  struct caldate cd;
  unsigned int numlen;

  if (argc < 2) return 1;

  while (*++argv) {
    if (scan_long(*argv, &year) != str_len(*argv)) continue;

    cd.year = year;
    cd.month = 1;
    cd.day = 1;
    daystart = caldate_mjd(&cd);
    cd.year = year + 1;
    dayend = caldate_mjd(&cd);

    while ((daystart + 3) % 7) --daystart;
    while ((dayend + 3) % 7) ++dayend;

    for (day = daystart; day < dayend; ++day) {
      caldate_frommjd(&cd, day, &weekday, (int *) 0);
      if (cd.year != year) {
        djbio_put(djbiofd_out, "   ", 3);
      }
      else {
        if (cd.month & 1) {
          if (cd.day < 10) {
            djbio_put(djbiofd_out, " ", 1);
          }
          else {
            numlen = fmt_ulong(strnum, (unsigned long)(cd.day / 10));
            djbio_put(djbiofd_out, strnum, numlen);
            djbio_put(djbiofd_out, "\010", 1);
            djbio_put(djbiofd_out, strnum, numlen);
          }
          numlen = fmt_ulong(strnum, (unsigned long)(cd.day % 10));
          djbio_put(djbiofd_out, strnum, numlen);
          djbio_put(djbiofd_out, "\010", 1);
          djbio_put(djbiofd_out, strnum, numlen);
        }
        else {
          numlen = fmt_ulong(strnum, (unsigned long)(cd.day));
          if (numlen < 2) {
            djbio_put(djbiofd_out, " ", 1);
          }
          djbio_put(djbiofd_out, strnum, numlen);
        }
        djbio_put(djbiofd_out, " ", 1);
        if (weekday == 6) {
          if ((cd.day >= 15) && (cd.day < 22)) {
            djbio_put(djbiofd_out, " ", 1);
            djbio_puts(djbiofd_out, montab[cd.month - 1]);
            djbio_put(djbiofd_out, " ", 1);
	    djbio_put(djbiofd_out, strnum, fmt_ulong(strnum, (unsigned long)year));
          }
          djbio_put(djbiofd_out, "\n", 1);
        }
      }
    }
    djbio_putflush(djbiofd_out, "\n", 1);
  }

  return 0;
}
